<?php

namespace Drp\UserManagement;

use Carbon\Carbon;
use Drp\UserManagement\Contracts\ManagedUser;
use Drp\UserManagement\Contracts\TokenGenerator;
use Drp\UserManagement\Events\UserCreated;
use Drp\UserManagement\Events\UserCreatedWithLoginCode;

class UserCreator
{
    /**
     * @var TokenGenerator
     */
    private $tokenGenerator;

    public function __construct(TokenGenerator $tokenGenerator)
    {
        $this->tokenGenerator = $tokenGenerator;
    }

    public static function getUserModel(): ManagedUser
    {
        return app(config('user_management.model'));
    }

    public function reIssueLoginCode(ManagedUser $user): ManagedUser
    {
        $user->addLoginToken(
            $this->tokenGenerator->create(),
            $this->getLoginExpiry()
        );

        event(new UserCreatedWithLoginCode($user));

        return $user;
    }

    public function createStandardUser(array $data = []): ManagedUser
    {
        $data['approved'] = config('user_management.requires_approval') ? 0 : 1;

        $user = $this
            ->getUserModel()
            ->create($data);

        event(new UserCreated($user));

        return $user;
    }

    public function createWithLoginCode(array $data = []): ManagedUser
    {
        $data['approved'] = 1;

        $user = $this
            ->getUserModel()
            ->create(array_merge($data, [
                'first_login_token' => $this->tokenGenerator->create(),
                'login_link_expiry' => $this->getLoginExpiry(),
            ]));

        event(new UserCreatedWithLoginCode($user));

        return $user;
    }

    protected function getLoginExpiry(): Carbon
    {
        return Carbon::now()->addDays(config('user_management.login_token_expiry_days'));
    }
}
